--[[
    TMemoryRecord の補助関数
]]
local memoryRecordHelper = {}

--[[
    第一引数に指定されたメモリーレコードにヘッダーを追加する。
    entry : 挿入先のメモリーレコード
    description : 追加するメモリーレコードの説明文
    return : 作成したヘッダーのメモリーレコードを返す。
]]
memoryRecordHelper.AddAsHeaderTo = function(entry, description, color)
    local mr = getAddressList().createMemoryRecord()
    mr.setDescription(description)
    mr.appendToEntry(entry)
    mr.IsGroupHeader = true
    mr.Options = "[moHideChildren]"
    mr.DontSave = true
    if color ~= nil then mr.Color = color end
    return mr
end

--[[
    第一引数に指定されたメモリーレコードにヘッダーを追加する。
    entry : 挿入先のメモリーレコード
    description : 追加するメモリーレコードの説明文
    return : 作成したヘッダーのメモリーレコードを返す。
]]
memoryRecordHelper.AddAsLineTo = function(entry, color)
    local mr = getAddressList().createMemoryRecord()
    mr.setDescription("====================================================================")
    mr.appendToEntry(entry)
    mr.IsGroupHeader = true
    mr.Options = "[moHideChildren]"
    mr.DontSave = true
    if color ~= nil then mr.Color = color end
    return mr
end

--[[
    第一引数に指定されたメモリーレコードに新たなメモリーレコードを追加する
    entry : 挿入先のメモリーレコード
    他の引数  : MemoryRecord クラスを参照 (https://wiki.cheatengine.org/index.php?title=Lua:Class:MemoryRecord)
    return : 作成したメモリーレコードを返す
]]
memoryRecordHelper.AddTo = function(entry, address, description, type, showAsSigned, text, dropDownReadOnly, dropDownDescriptionOnly, displayAsDropDownListItem)
    local mr = getAddressList().createMemoryRecord()
    mr.SetAddress(address)
    mr.setDescription(description)
    mr.appendToEntry(entry)
    mr.DontSave = true
    if type ~= nil then mr.Type = type end
    if showAsSigned ~= nil then mr.ShowAsSigned = showAsSigned end

    if text ~= nil then
        mr.DropDownList.Text = text
        if dropDownReadOnly ~= nil then mr.DropDownReadOnly = dropDownReadOnly end
        if dropDownDescriptionOnly ~= nil then mr.DropDownDescriptionOnly = dropDownDescriptionOnly end
        if displayAsDropDownListItem ~= nil then mr.DisplayAsDropDownListItem = displayAsDropDownListItem end
    end

    return mr
end

--[[
    AddressList から指定された Description と一致するメモリーレコードを検索し、最初に見つかったメモリーレコードを返す。
    ... : Description1, Description2, ...
    return : 見つかったらメモリーレコードを返し、見つからなければ nil を返す。
]]
memoryRecordHelper.GetByDescriptions = function(...)
    local memrec = getAddressList().getMemoryRecordByDescription((select(1, ...)))
    if not memrec then return nil end

    for i = 2, select("#", ...) do
        local description = (select(i, ...))
        local isFound = false
        for j = 0, memrec.Count - 1 do
            if description == memrec.Child[j].Description then
                memrec = memrec.Child[j]
                isFound = true
                break
            end
        end

        if isFound == false then return nil end
    end

    return memrec
end

--[[
    メモリーレコードの色を RGB で設定する。
    r, g, b : 0 ~ 255 の整数値
]]
memoryRecordHelper.SetRGBColor = function(target, r, g, b)
    if target == nil then return end
    target.Color = r + (g << 8) + (b << 16)
end

--[[
    指定されたメモリーレコードのすべての子要素を削除する
]]
memoryRecordHelper.RemoveAllChildren = function(parent)
    if parent == nil then return end
    while parent.Count > 0 do
        memoryrecord_delete(parent.Child[parent.Count - 1])
    end
end

return memoryRecordHelper